import { Injectable } from '@angular/core';
import { LangChangeEvent, TranslateService } from '@ngx-translate/core';
import { Subject } from 'rxjs';
import { Themes } from 'src/app/models/view/themes.enum';
import { AuthService } from '../auth/auth.service';

@Injectable({
  providedIn: 'root'
})
export class ThemeService {

  themeName: string = Themes.Default;

  sidebarWidth = '150px';
  isSidebarOpen = false;
  hasLangChanged = false;

  themeName$ = new Subject<string>();
  isSidebarOpen$ = new Subject<boolean>();
  sidebarWidth$ = new Subject<string>();

  constructor(
    private authService: AuthService
  ) {
    this.themeName$.subscribe(themeName => {
      this.themeName = themeName;
    });
    this.isSidebarOpen$.subscribe(isSidebarOpen => {
      this.isSidebarOpen = isSidebarOpen;
      this.sidebarWidth$.next(isSidebarOpen ? '175px' : '75px');
    });
    this.sidebarWidth$.subscribe(sidebarWidth => {
      this.sidebarWidth = sidebarWidth;
    });

    this.loadTheme();
  }

  loadTheme(): void {
    const token = this.authService.getDecodedToken();
    const mail = this.authService.getEMailFromDecodedToken(token);

    let defaultThemeName = localStorage.getItem(`default_themeName`);
    if (!defaultThemeName || defaultThemeName != Themes.Default) {
      defaultThemeName = Themes.Default;
    }

    if (!mail) {
      this.setTheme(defaultThemeName);
      return;
    }

    let userThemeName = localStorage.getItem(`${mail}_themeName`);
    if (!userThemeName) {
      this.setTheme(defaultThemeName);
      return;
    }
    this.setTheme(userThemeName);
  }

  setTheme(name: string): void {
    this.authService.isUserLoggedInAsync().then(result => {
      if (!result) {
        localStorage.setItem(`default_themeName`, Themes.Default);
        this.themeName$.next(Themes.Default);
      }

      const token = this.authService.getDecodedToken();
      const mail = this.authService.getEMailFromDecodedToken(token);
      if (mail) {
        localStorage.setItem(`${mail}_themeName`, name);
      }
      localStorage.setItem(`default_themeName`, name);
      this.themeName$.next(name);
    });
  }

  loadMenu() {
    const token = this.authService.getDecodedToken();
    const mail = this.authService.getEMailFromDecodedToken(token);

    if (!mail) {
      this.setSideWidth(true);
      return;
    }

    let isMenuOpen = true;
    let isMenuOpenStr = localStorage.getItem(`${mail}_isMenuOpen`);
    if (isMenuOpenStr) {
      isMenuOpen = JSON.parse(isMenuOpenStr);
    }

    this.setIsMenuOpen(isMenuOpen);
  }

  setIsMenuOpen(isMenuOpen: boolean) {
    const token = this.authService.getDecodedToken();
    const mail = this.authService.getEMailFromDecodedToken(token);

    if (!mail) {
      this.setSideWidth(true);
      return;
    }

    localStorage.setItem(`${mail}_isMenuOpen`, isMenuOpen + "");
    this.setSideWidth(isMenuOpen);
  }

  setSideWidth(isSidebarOpen: boolean): void {
    this.isSidebarOpen$.next(isSidebarOpen);
  }
}
